/***
 * PictureFrame.java,v 1.4 1997/07/07 20:13:19 dfs Exp
 * Copyright 1997 Original Reusable Objects, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, US
 ***/

import java.awt.*;
import java.applet.*;

import com.oroinc.yaray.*;

/***
 *
 * <p>
 * Copyright &#169 1996, 1997 Original Reusable Objects, Inc.
 * All rights reserved.
 * <p>
 * @author Daniel F. Savarese
 ***/

public final class PictureFrame extends Frame implements Runnable {
  Scene _scene;
  PictureCanvas _picture;
  Panel _picturePanel;
  Thread _raytrace;
  boolean _destroyed = false, _isApplication;

  public PictureFrame(Scene scene, boolean isApplication) {
    GridBagConstraints constraints;
    GridBagLayout layout;

    _isApplication = isApplication;
    _picturePanel = new Panel();
    _picture      = new PictureCanvas();

    _scene           = scene;
    constraints      = new GridBagConstraints();
    setLayout(layout = new GridBagLayout());

    _picturePanel.setLayout(layout);

    _picture.resize(_scene.getWidth(), _scene.getHeight());
    constraints.weightx = 0.0;
    constraints.weighty = 0.0;
    constraints.fill    = GridBagConstraints.BOTH;
    constraints.anchor  = GridBagConstraints.CENTER;
    constraints.gridwidth = GridBagConstraints.REMAINDER;
    layout.setConstraints(_picture, constraints);
    _picturePanel.add(_picture);

    constraints.weightx = 1.0;
    constraints.weighty = 1.0;
    constraints.fill    = GridBagConstraints.BOTH;
    constraints.anchor  = GridBagConstraints.CENTER;
    constraints.gridwidth = GridBagConstraints.REMAINDER;
    layout.setConstraints(_picturePanel, constraints);
    add(_picturePanel);
  }

  public boolean destroyed() { return _destroyed; }

  public boolean handleEvent(Event event) {

    if(event.id == Event.WINDOW_DESTROY) {
      dispose();
      if(_isApplication)
	System.exit(1);
      return true;
    }

    return super.handleEvent(event);
  }


  public void start() {
    if(_raytrace != null) {
      if(_raytrace.isAlive()) {
	_raytrace.resume();
	System.err.println("Raytracing resumed.");
      }
      return;
    }

    _raytrace = new Thread(this);
    _raytrace.start();
  }


  public void stop() {
    if(_raytrace != null && _raytrace.isAlive()) {
      _raytrace.suspend();
      System.err.println("Raytracing suspended.");
    }
  }


  public void destroy() {
    if(_destroyed)
      return;
    hide();
    if(_raytrace != null && _raytrace.isAlive())
      _raytrace.stop();
    _destroyed = true;
  }


  public void dispose() {
    destroy();
    //super.dispose();
  }


  public void run() {
    Raytracer tracer;

    _picture.setDimensions(_scene.getWidth(), _scene.getHeight());
    tracer = new Raytracer();
    tracer.raytraceScene(_scene, _picture);

    System.err.println("Raytracing finished.");
  }


}
