/***
 * view.java,v 1.3 1997/07/07 20:13:19 dfs Exp
 * Copyright 1997 Original Reusable Objects, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, US
 ***/

import java.io.*;

import com.oroinc.math.geometry.*;
import com.oroinc.yaray.*;
import com.oroinc.text.regex.*;
import com.oroinc.text.perl.*;

/***
 * This program generates reads a scene file as input and either
 * displays the scene in a window frame as it is being generated or
 * writes it to a PPM format file if an outputfile argument is provided.
 * <p>
 * Copyright &#169 1996, 1997 Original Reusable Objects, Inc.
 * All rights reserved.
 * <p>
 * @author Daniel F. Savarese
 ***/

public final class view {
  static final String headerExpression =
"/SceneDimensions:\\s+(\\d+)\\s+(\\d+)\\s+" +
"Eye:\\s+(-?\\d*\\.?\\d+)\\s+(-?\\d*\\.?\\d+)\\s+(-?\\d*\\.?\\d+)\\s+" +
"ViewingDirection:\\s+(-?\\d*\\.?\\d+)\\s+(-?\\d*\\.?\\d+)\\s+(-?\\d*\\.?\\d+)\\s+" +
"UpVector:\\s+(-?\\d*\\.?\\d+)\\s+(-?\\d*\\.?\\d+)\\s+(-?\\d*\\.?\\d+)\\s+" +
"WindowDimensions:\\s+(-?\\d*\\.?\\d+)\\s+(-?\\d*\\.?\\d+)/i";
  static final String lightExpression =
"/Light\\s+\\d+:\\s+" +
"Location:\\s+(-?\\d*\\.?\\d+)\\s+(-?\\d*\\.?\\d+)\\s+(-?\\d*\\.?\\d+)\\s+" +
"Color:\\s+(\\d*\\.?\\d+)\\s+(\\d*\\.?\\d+)\\s+(\\d*\\.?\\d+)/i";
  static final String finishExpression =
"/Finish\\s+\\d+:\\s+" +
"AmbientConstant:\\s+(-?\\d*\\.?\\d+)\\s+" +
"DiffuseConstant:\\s+(-?\\d*\\.?\\d+)\\s+" +
"BrillianceConstant:\\s+(-?\\d*\\.?\\d+)\\s+" +
"SpecularConstant:\\s+(-?\\d*\\.?\\d+)\\s+" +
"RoughnessConstant:\\s+(-?\\d*\\.?\\d+)\\s+" +
"ReflectivityConstant:\\s+(-?\\d*\\.?\\d+)\\s+" +
"TransmissionConstant:\\s+(-?\\d*\\.?\\d+)\\s+" +
"RefractionConstant:\\s+(-?\\d*\\.?\\d+)/i";

  static final String pigmentHeader =
"/Pigment\\s+\\d+:\\s+" +
"Type:\\s+(\\w+)/i";

  static final String solidExpression =
"/Color:\\s+(\\d*\\.?\\d+)\\s+(\\d*\\.?\\d+)\\s+(\\d*\\.?\\d+)/i";

  static final String checkerExpression =
"/Color1:\\s+(\\d*\\.?\\d+)\\s+(\\d*\\.?\\d+)\\s+(\\d*\\.?\\d+)\\s+" +
"Color2:\\s+(\\d*\\.?\\d+)\\s+(\\d*\\.?\\d+)\\s+(\\d*\\.?\\d+)\\s+" +
"Length:\\s+(\\d*\\.?\\d+)/i";

  static final String gradientExpression =
"/GradientVector:\\s+(-?\\d*\\.?\\d+)\\s+(-?\\d*\\.?\\d+)\\s+(-?\\d*\\.?\\d+)\\s+" +
"Color1:\\s+(\\d*\\.?\\d+)\\s+(\\d*\\.?\\d+)\\s+(\\d*\\.?\\d+)\\s+" +
"Color2:\\s+(\\d*\\.?\\d+)\\s+(\\d*\\.?\\d+)\\s+(\\d*\\.?\\d+)/i";

  static final String objectHeader =
"/Object\\s+\\d+:\\s+" +
"Type:\\s+(\\w+)/i";

  static final String sphereExpression =
"/Center:\\s+(-?\\d*\\.?\\d+)\\s+(-?\\d*\\.?\\d+)\\s+(-?\\d*\\.?\\d+)\\s+" +
"Radius:\\s+(\\d*\\.?\\d+)\\s+" +
"Pigment:\\s+(\\d+)\\s+" +
"Finish:\\s+(\\d+)/i";

  static final String discExpression =
"/Center:\\s+(-?\\d*\\.?\\d+)\\s+(-?\\d*\\.?\\d+)\\s+(-?\\d*\\.?\\d+)\\s+" +
"Normal:\\s+(-?\\d*\\.?\\d+)\\s+(-?\\d*\\.?\\d+)\\s+(-?\\d*\\.?\\d+)\\s+" +
"Radius:\\s+(\\d*\\.?\\d+)\\s+" +
"Pigment:\\s+(\\d+)\\s+" +
"Finish:\\s+(\\d+)/i";

  static Perl5Util perl = new Perl5Util();

  public static void setTuple(Tuple tuple,
			      String val1, String val2, String val3) 
  {
    tuple.setElement(0, Float.valueOf(val1).floatValue());
    tuple.setElement(1, Float.valueOf(val2).floatValue());
    tuple.setElement(2, Float.valueOf(val3).floatValue());
  }

  public static void readScene(Scene scene, InputStream stream)
       throws IOException, NumberFormatException
  {
    int num;
    Perl5StreamInput input;
    String string;

    input = new Perl5StreamInput(stream);

    if(perl.match(headerExpression, input)) {
      scene.sceneWidth  = Integer.parseInt(perl.group(1));
      scene.sceneHeight = Integer.parseInt(perl.group(2));
      setTuple(scene.eye, perl.group(3), perl.group(4), perl.group(5));
      setTuple(scene.viewingDirection,
	       perl.group(6), perl.group(7), perl.group(8));
      setTuple(scene.upVector, perl.group(9), perl.group(10), perl.group(11));
      scene.windowWidth  = 
	  Float.valueOf(perl.group(12)).floatValue();
      scene.windowHeight = 
	  Float.valueOf(perl.group(13)).floatValue();
    } else
      throw new IOException("Incorrectly formatted scene description.");

    if(perl.match("/Lights:\\s+(\\d+)/i", input))
      scene.lights = new Light[Integer.parseInt(perl.group(1))];
    else
      throw new IOException("Bad lights specification.");

    for(num = 0; num < scene.lights.length; num++) {
      if(perl.match(lightExpression, input)) {
	scene.lights[num] = new Light();
	setTuple(scene.lights[num].location,
		 perl.group(1), perl.group(2), perl.group(3));
	setTuple(scene.lights[num].color,
		 perl.group(4), perl.group(5), perl.group(6));
      } else
	throw new IOException("Bad lights specification.  Light #" + num);
    }

    if(perl.match("/Finishes:\\s+(\\d+)/i", input))
      scene.finishes = new Finish[Integer.parseInt(perl.group(1))];
    else
      throw new IOException("Bad finishes specification.");

    for(num = 0; num < scene.finishes.length; num++) {
      if(perl.match(finishExpression, input)) {
	scene.finishes[num] = new Finish();
	scene.finishes[num].kAmbient      = 
	  Float.valueOf(perl.group(1)).floatValue();
	scene.finishes[num].kDiffuse      =
	  Float.valueOf(perl.group(2)).floatValue();
	scene.finishes[num].brilliance    =
	  Float.valueOf(perl.group(3)).floatValue();
	scene.finishes[num].kSpecular     =
	  Float.valueOf(perl.group(4)).floatValue();
	scene.finishes[num].roughness     =
	  Float.valueOf(perl.group(5)).floatValue();
	scene.finishes[num].kReflectivity =
	  Float.valueOf(perl.group(6)).floatValue();
	scene.finishes[num].kTransmitted  =
	  Float.valueOf(perl.group(7)).floatValue();
	scene.finishes[num].nRefraction   =
	  Float.valueOf(perl.group(8)).floatValue();
      } else
	throw new IOException("Bad finish specification.  Finish #" + num);
    }

    if(perl.match("/Pigments:\\s+(\\d+)/i", input))
      scene.pigments = new Pigment[Integer.parseInt(perl.group(1))];
    else
      throw new IOException("Bad pigments specification.");

    for(num = 0; num < scene.pigments.length; num++) {

      if(perl.match(pigmentHeader, input)) {
	string = perl.group(1);
	if(string.equalsIgnoreCase("solid")) {
	  Solid pigment = new Solid();

	  if(perl.match(solidExpression, input)) {
	    setTuple(pigment, perl.group(1), perl.group(2), perl.group(3));
	    scene.pigments[num] = pigment;
	  } else
	    throw
	      new IOException("Bad solid pigment specification.  Pigment #" +
			      num);
	} else if(string.equalsIgnoreCase("checker")) {
	  Checkerboard pigment = new Checkerboard();

	  if(perl.match(checkerExpression, input)) {
	    setTuple(pigment.color1,
		     perl.group(1), perl.group(2), perl.group(3));
	    setTuple(pigment.color2,
		     perl.group(4), perl.group(5), perl.group(6));
	    pigment.sideLength =
	      Float.valueOf(perl.group(7)).floatValue();
	    scene.pigments[num] = pigment;
	  } else
	    throw
	      new IOException("Bad checker pigment specification.  Pigment #" +
			      num);

	} else if(string.equalsIgnoreCase("gradient")) {
	  Gradient pigment = new Gradient();


	  if(perl.match(gradientExpression, input)) {
	    setTuple(pigment.gradient,
		     perl.group(1), perl.group(2), perl.group(3));
	    setTuple(pigment.color1,
		     perl.group(4), perl.group(5), perl.group(6));
	    setTuple(pigment.color2,
		     perl.group(7), perl.group(8), perl.group(9));
	    scene.pigments[num] = pigment;
	  } else
	    throw
	      new IOException("Bad gradient pigment specification.  Pigment #" +
			      num);
	} else
	  throw new IOException("Bad pigment specification.  Pigment #" + num);
      } else
	throw new IOException("Bad pigment specification.  Pigment #" + num);
    }

    if(perl.match("/Objects:\\s+(\\d+)/i", input))
      scene.objects = new WorldObject[Integer.parseInt(perl.group(1))];
    else
      throw new IOException("Bad objects specification.");

    for(num = 0; num < scene.objects.length; num++) {
      int finish, pigment;
      float tmpFloat;
      Point tmpPoint            = new Point(3);
      GeometricVector tmpVector = new GeometricVector(3);

      if(perl.match(objectHeader, input)) {
	string = perl.group(1);
	if(string.equalsIgnoreCase("sphere")) {

	  if(perl.match(sphereExpression, input)) {
	    setTuple(tmpPoint, perl.group(1), perl.group(2), perl.group(3));

	    tmpFloat = Float.valueOf(perl.group(4)).floatValue();
	    finish   = Integer.parseInt(perl.group(5));
	    pigment  = Integer.parseInt(perl.group(6));

	    scene.objects[num] =
	      new Sphere(tmpPoint, tmpFloat, scene.pigments[pigment],
			 scene.finishes[finish]);
	  } else 
	    throw
	      new IOException("Bad sphere specification.  Object #" + num);
	} else if(string.equalsIgnoreCase("disc")) {

	  if(perl.match(discExpression, input)) {
	    setTuple(tmpPoint, perl.group(1), perl.group(2), perl.group(3));
	    setTuple(tmpVector, perl.group(4), perl.group(5), perl.group(6));

	    tmpFloat = Float.valueOf(perl.group(7)).floatValue();
	    finish   = Integer.parseInt(perl.group(8));
	    pigment  = Integer.parseInt(perl.group(9));

	    scene.objects[num] =
	      new Disc(tmpVector, tmpPoint, tmpFloat,
		       scene.pigments[pigment], scene.finishes[finish]);
	  } else
	    throw
	      new IOException("Bad disc specification.  Object #" + num);
	} else
	  throw new IOException("Unrecognized object type.  Object #" + num);
      } else
	throw new IOException("Bad object specification.  Object #" + num);
    }
  }

  public final static void main(String[] args) {
    Scene scene;
    InputStream input = null;
    OutputStream output;

    if(args.length < 1) {
      System.err.println("Usage: view sceneFile [ppmOutputFile]");
      System.exit(1);
    } 

    try {
      input = new BufferedInputStream(new FileInputStream(args[0]));
    } catch(IOException e) {
      System.err.println("Could not open file: " + args[0]);
      e.printStackTrace();
      System.exit(1);
    }

    scene   = new Scene();

    try {
      readScene(scene, input);
    } catch(Exception e) {
      System.err.println("I/O error occurred while reading scene.");
      e.printStackTrace();
      System.exit(1);
    }

    if(args.length == 1) {
      PictureFrame frame = new PictureFrame(scene, true);
      frame.pack();
      frame.show();
      frame.start();
    } else {
      Raytracer tracer = new Raytracer();
      PPMPicture picture = new PPMPicture();

      picture.setDimensions(scene.getWidth(), scene.getHeight());
      tracer.raytraceScene(scene, picture);

      try {
	output = new BufferedOutputStream(new FileOutputStream(args[1]));
	picture.write(output);
	output.close();
      } catch(IOException e) {
	System.err.println("I/O error occurred while writing output file.");
	e.printStackTrace();
	System.exit(1);
      }
    }


  }

}
