/***
 * PPMPicture.java,v 1.4 1997/07/07 20:13:23 dfs Exp
 * Copyright 1997 Original Reusable Objects, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, US
 ***/
package com.oroinc.yaray;

import java.io.*;

import com.oroinc.math.geometry.*;

/***
 * Implements the Picture interface for PPM format files.
 * <p>
 * Copyright &#169 1996, 1997 Original Reusable Objects, Inc.
 * All rights reserved.
 * <p>
 * @author Daniel F. Savarese
 ***/

public class PPMPicture implements Picture {
  int _width, _height;
  byte[] _bytes;
  String _header;

  public void setDimensions(int width, int height) {
    _width  = width;
    _height = height;
    _bytes  = new byte[3*width*height];
    _header = "P6\n" + width + " " + height + "\n255\n";
  }


  public void setPixel(int x, int y, Color color) {
    int offset, red, green, blue;

    if((red = (int)(255*color.element(Color.RED))) > 255)
      red = 255;
    else if(red < 0)
      red = 0;

    if((green = (int)(255*color.element(Color.GREEN))) > 255)
      green = 255;
    else if(green < 0)
      green = 0;


    if((blue = (int)(255*color.element(Color.BLUE))) > 255)
      blue = 255;
    else if(green < 0)
      blue = 0;

    offset = 3*(y*_width + x);
    _bytes[offset]   = (byte)red;
    _bytes[++offset] = (byte)green;
    _bytes[++offset] = (byte)blue;
  }


  public void write(OutputStream output) throws IOException {
    DataOutputStream dout = new DataOutputStream(output);

    dout.writeBytes(_header);
    dout.write(_bytes);

  }

}
