/***
 * Ray.java,v 1.4 1997/07/07 20:13:23 dfs Exp
 * Copyright 1997 Original Reusable Objects, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, US
 ***/
package com.oroinc.yaray;

import com.oroinc.math.geometry.*;

/***
 * This class represents a ray that is traced from a point of origin
 * int the world.
 * <p>
 * Copyright &#169 1996, 1997 Original Reusable Objects, Inc.
 * All rights reserved.
 * <p>
 * @author Daniel F. Savarese
 ***/

public class Ray {
  /*** The origin of the ray. ***/
  public Point origin;
  /*** The direction the ray is being traced. ***/
  public GeometricVector direction;
  /*** The current length of the ray as it is being traced. ***/
  public float extent;

  /***
   * Creates a ray of extent 0 with origin (0, 0, 0) and no direction
   * (i.e., a vector pointing nowhere [0, 0, 0]).
   ***/
  public Ray() {
    origin    = new Point(3);
    direction = new GeometricVector(3);
    extent    = 0;
  }

  /***
   * Returns the endpoint of the ray.
   * <p>
   * @return The point defining the location of the end of the ray.
   ***/
  public Point endpoint() {
    Point result;
    result = new Point(direction.multiply(extent));
    result.addTo(origin);
    return result;
  }
}

