/***
 * Raytracer.java,v 1.5 1997/07/07 20:13:23 dfs Exp
 * Copyright 1997 Original Reusable Objects, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, US
 ***/
package com.oroinc.yaray;

import com.oroinc.math.geometry.*;

/***
 * A raytracing engine.
 * <p>
 * Copyright &#169 1996, 1997 Original Reusable Objects, Inc.
 * All rights reserved.
 * <p>
 * @author Daniel F. Savarese
 ***/

public final class Raytracer {

  public void raytraceScene(Scene scene, Picture picture) {
    int i, j;
    float x, y, dx, dy;
    IntersectionRay ray = new IntersectionRay();
    Point tmpPoint = new Point(3);
    Frame viewFrame = new Frame(3);
    GeometricVector zViewBasis, xViewBasis, yViewBasis;
    World world = new World();
    Color color = new Color();

    world.lights     = scene.lights;
    world.objects    = scene.objects;
    world.eye        = scene.eye;

    // It would be cleaner to put this stuff into a camera object
    // Set up view frame
    zViewBasis = scene.viewingDirection.normalized();
    xViewBasis = GeometricVector.cross(zViewBasis, scene.upVector);
    xViewBasis.normalize();
    yViewBasis = GeometricVector.cross(xViewBasis, zViewBasis);
    yViewBasis.normalize();
    viewFrame.setOrigin(scene.eye);
    viewFrame.setBasisVector(1, xViewBasis);
    viewFrame.setBasisVector(2, yViewBasis);
    viewFrame.setBasisVector(3, zViewBasis);
    viewFrame.commit();

    dx = scene.windowWidth/scene.sceneWidth;
    dy = scene.windowHeight/scene.sceneHeight;

    ray.origin.assign(scene.eye);

  // For each pixel, generate a ray from the eye to the pixel.  Trace
  // this ray through the world and store the resulting color in
  // the final image.
    for(j = 0, y = 0.5f*scene.windowHeight;
	j < scene.sceneHeight; j++, y-=dy){
      for(i = 0, x = -0.5f*scene.windowWidth; i < scene.sceneWidth;
	  i++, x+=dx){
	ray.extent = Util.INFINITY;
	tmpPoint.setElement(0, x);
	tmpPoint.setElement(1, y);
	tmpPoint.setElement(2, 1);
	viewFrame.convertToStandardFrame(tmpPoint);
	ray.direction =
	  new GeometricVector(Tuple.subtract(tmpPoint, scene.eye));
	ray.direction.normalize();
	world.raytrace(ray, color);
	picture.setPixel(i, j, color); 
      }
    }
  }

}




