/***
 * World.java,v 1.4 1997/07/07 20:13:24 dfs Exp
 * Copyright 1997 Original Reusable Objects, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version. 
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. 
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, US
 ***/
package com.oroinc.yaray;

import com.oroinc.math.geometry.*;

/***
 * The world through which rays are traced to generate a picture.
 * <p>
 * Copyright &#169 1996, 1997 Original Reusable Objects, Inc.
 * All rights reserved.
 * <p>
 * @author Daniel F. Savarese
 ***/

public class World {
  /*** Maximum recursion depth.  Limits the number of reflections. ***/
  public static final int MAX_DEPTH = 4;
  public static final float EPSILON = 1e-6f;

  public int currentDepth;
  public Point eye;
  public WorldObject objects[];
  public Light[] lights;
  
  public World() {
    eye     = new Point(3);
    currentDepth = 0;
    lights  = null;
    objects = null;
  }


  /***
   * Trace the ray, checking for intersection with each object
   * in the world.  If there is an intersection, compute the
   * color at the point of intersection, otherwise return black.
   * The color is returned via the color reference parameter.
   * This method can only be called recursively a maximum of MAX_DEPTH
   * times for any instance of a World object.
   ***/

  public void raytrace(IntersectionRay ray, Color color) {
    int object;
    Point tmpEye = new Point(3);
    Color defaultColor = new Color(0, 0, 0);

    // Check if maximum recursion depth is exceeded
    if(++currentDepth > World.MAX_DEPTH){
      color = defaultColor;
      --currentDepth;
      return;
    }

    ray.intersection = null;

    for(object = 0; object < objects.length; object++)
      objects[object].intersect(ray);

    if(ray.intersection != null){
      tmpEye.assign(eye);
      eye.assign(ray.origin);
      ray.intersection.getColor(this, ray, color);
      eye.assign(tmpEye);
    } else
      color.assign(defaultColor);

    --currentDepth;
  }

}
